=== modified file 'bashcomp.py'
--- bashcomp.py	2010-04-30 15:46:39 +0000
+++ bashcomp.py	2010-04-30 17:24:59 +0000
@@ -175,6 +175,7 @@
             bzr_version += " and the following plugins:"
             for name, plugin in sorted(self.data.plugins.iteritems()):
                 bzr_version += "\n# %s" % plugin
+        return bzr_version
 
     def global_options(self):
         return " ".join(sorted(self.data.global_options))
@@ -312,7 +313,7 @@
         if plugin_name is not None:
             if (self.selected_plugins is not None and
                 plugin not in self.selected_plugins):
-                return
+                return None
             plugin_data = self.data.plugins.get(plugin_name)
             if plugin_data is None:
                 plugin_data = PluginData(plugin_name)
@@ -339,6 +340,8 @@
             cmd_data.fixed_words = ('"$cmds %s"' %
                 " ".join(sorted(help_topics.topic_registry.keys())))
 
+        return cmd_data
+
     def option(self, opt):
         optswitches = {}
         parser = option.get_optparser({opt.name: opt})

=== modified file 'tests/test_bashcomp.py'
--- tests/test_bashcomp.py	2010-04-30 08:27:50 +0000
+++ tests/test_bashcomp.py	2010-04-30 17:24:59 +0000
@@ -17,7 +17,8 @@
 from bzrlib.tests import TestCase, TestCaseWithTransport, Feature
 from bzrlib import commands
 from StringIO import StringIO
-from ..bashcomp import bash_completion_function
+from ..bashcomp import *
+import bzrlib
 import os
 import subprocess
 
@@ -149,7 +150,7 @@
 
     def test_global_opts(self):
         c = self.complete(['bzr', '-', 'init'], cword=1,
-                          contains=['--no-plugins', '-?'])
+                          contains=['--no-plugins', '--builtin'])
 
     def test_commit_dashm(self):
         c = self.complete(['bzr', 'commit', '-m'], expect=['-m'])
@@ -201,3 +202,118 @@
         wt.branch.tags.set_tag('3tag', 'null:')
         self.complete(['bzr', 'log', '-r', 'tag', ':', 't'],
                       expect=['tag1', 'tag2'])
+
+
+class TestBashCodeGen(TestCase):
+
+    def test_command_names(self):
+        data = CompletionData()
+        bar = CommandData('bar')
+        bar.aliases.append('baz')
+        data.commands.append(bar)
+        data.commands.append(CommandData('foo'))
+        cg = BashCodeGen(data)
+        self.assertEqual('bar baz foo', cg.command_names())
+
+    def test_debug_output(self):
+        data = CompletionData()
+        self.assertEqual('', BashCodeGen(data, debug=False).debug_output())
+        self.assertTrue(BashCodeGen(data, debug=True).debug_output())
+
+    def test_bzr_version(self):
+        data = CompletionData()
+        cg = BashCodeGen(data)
+        self.assertEqual('%s.' % bzrlib.version_string, cg.bzr_version())
+        data.plugins['foo'] = PluginData('foo', '1.0')
+        data.plugins['bar'] = PluginData('bar', '2.0')
+        cg = BashCodeGen(data)
+        self.assertEqual('''\
+%s and the following plugins:
+# bar 2.0
+# foo 1.0''' % bzrlib.version_string, cg.bzr_version())
+
+    def test_global_options(self):
+        data = CompletionData()
+        data.global_options.add('--foo')
+        data.global_options.add('--bar')
+        cg = BashCodeGen(data)
+        self.assertEqual('--bar --foo', cg.global_options())
+
+    def test_command_cases(self):
+        data = CompletionData()
+        bar = CommandData('bar')
+        bar.aliases.append('baz')
+        bar.options.append(OptionData('--opt'))
+        data.commands.append(bar)
+        data.commands.append(CommandData('foo'))
+        cg = BashCodeGen(data)
+        self.assertEqualDiff('''\
+\tbar|baz)
+\t\tcmdOpts='--opt'
+\t\t;;
+\tfoo)
+\t\tcmdOpts=''
+\t\t;;
+''', cg.command_cases())
+
+    def test_command_case(self):
+        cmd = CommandData('cmd')
+        cmd.plugin = PluginData('plugger', '1.0')
+        bar = OptionData('--bar')
+        bar.registry_keys = ['that', 'this']
+        bar.error_messages.append('Some error message')
+        cmd.options.append(bar)
+        cmd.options.append(OptionData('--foo'))
+        data = CompletionData()
+        data.commands.append(cmd)
+        cg = BashCodeGen(data)
+        self.assertEqualDiff('''\
+\tcmd)
+\t\t# plugin "plugger 1.0"
+\t\t# Some error message
+\t\tcmdOpts='--bar=that --bar=this --foo'
+\t\tcase $curOpt in
+\t\t\t--bar) optEnums='that this' ;;
+\t\tesac
+\t\t;;
+''', cg.command_case(cmd))
+
+
+class TestDataCollector(TestCase):
+
+    def setUp(self):
+        super(TestDataCollector, self).setUp()
+        commands.install_bzr_command_hooks()
+
+    def test_global_options(self):
+        dc = DataCollector()
+        dc.global_options()
+        self.assertSubset(['--no-plugins', '--builtin'],
+                           dc.data.global_options)
+
+    def test_commands(self):
+        dc = DataCollector()
+        dc.commands()
+        self.assertSubset(['init', 'init-repo', 'init-repository'],
+                           dc.data.all_command_aliases())
+
+    def test_commit_dashm(self):
+        dc = DataCollector()
+        cmd = dc.command('commit')
+        self.assertSubset(['-m'],
+                           [str(o) for o in cmd.options])
+
+    def test_status_negated(self):
+        dc = DataCollector()
+        cmd = dc.command('status')
+        self.assertSubset(['--no-versioned', '--no-verbose'],
+                           [str(o) for o in cmd.options])
+
+    def test_init_format(self):
+        dc = DataCollector()
+        cmd = dc.command('init')
+        for opt in cmd.options:
+            if opt.name == '--format':
+                self.assertSubset(['2a'], opt.registry_keys)
+                return
+        raise AssertionError('Option --format not found')

